@@ -86,7 +86,7 @@ async fn pair(app: &mut App, sender: UnboundedSender<Event>) {
                 Ok(device) => match device.alias().await {
                     Ok(device_name) => {
                         let _ = Notification::send(
-                            format!("Start pairing with the device\n `{device_name}`",),
+                            format!("Start pairing with the device {device_name}",),
                             NotificationLevel::Info,
                             sender.clone(),
                         );
@@ -99,6 +99,40 @@ async fn pair(app: &mut App, sender: UnboundedSender<Event>) {
                                         NotificationLevel::Info,
                                         sender.clone(),
                                     );
+
+                                    let _ = sender.send(Event::NewPairedDevice(device.address()));
+                                    match device.set_trusted(true).await {
+                                        Ok(_) => {
+                                            let _ = Notification::send(
+                                                "Device trusted".to_string(),
+                                                NotificationLevel::Info,
+                                                sender.clone(),
+                                            );
+                                        }
+                                        Err(e) => {
+                                            let _ = Notification::send(
+                                                e.to_string(),
+                                                NotificationLevel::Error,
+                                                sender.clone(),
+                                            );
+                                        }
+                                    };
+                                    match device.connect().await {
+                                        Ok(_) => {
+                                            let _ = Notification::send(
+                                                "Device connected".to_string(),
+                                                NotificationLevel::Info,
+                                                sender.clone(),
+                                            );
+                                        }
+                                        Err(e) => {
+                                            let _ = Notification::send(
+                                                e.to_string(),
+                                                NotificationLevel::Error,
+                                                sender.clone(),
+                                            );
+                                        }
+                                    };
                                 }
                                 Err(e) => {
                                     let _ = Notification::send(
@@ -106,6 +140,7 @@ async fn pair(app: &mut App, sender: UnboundedSender<Event>) {
                                         NotificationLevel::Error,
                                         sender.clone(),
                                     );
+                                    let _ = sender.send(Event::FailedPairing(device.address()));
                                 }
                             }
                         });
@@ -171,6 +206,66 @@ pub async fn handle_key_events(
                     .handle_event(&crossterm::event::Event::Key(key_event));
             }
         },
+        FocusedBlock::RequestConfirmation => match key_event.code {
+            KeyCode::Tab => {
+                if let Some(confirmation) = &mut app.requests.confirmation {
+                    confirmation.toggle_select();
+                }
+            }
+            KeyCode::Esc => {
+                if let Some(confirmation) = &mut app.requests.confirmation {
+                    confirmation.cancel(&app.auth_agent).await?;
+                }
+            }
+            KeyCode::Enter => {
+                if let Some(confirmation) = &mut app.requests.confirmation {
+                    confirmation.submit(&app.auth_agent).await?;
+                }
+            }
+
+            _ => {}
+        },
+        FocusedBlock::EnterPinCode => {
+            if let Some(req) = &mut app.requests.enter_pin_code {
+                match key_event.code {
+                    KeyCode::Esc => {
+                        req.cancel(&app.auth_agent).await?;
+                    }
+
+                    _ => {
+                        req.handle_key_events(key_event, &app.auth_agent).await?;
+                    }
+                }
+            }
+        }
+        FocusedBlock::EnterPasskey => {
+            if let Some(req) = &mut app.requests.enter_passkey {
+                match key_event.code {
+                    KeyCode::Esc => {
+                        req.cancel(&app.auth_agent).await?;
+                    }
+
+                    _ => {
+                        req.handle_key_events(key_event, &app.auth_agent).await?;
+                    }
+                }
+            }
+        }
+        FocusedBlock::DisplayPinCode => {
+            if let Some(req) = &mut app.requests.display_pin_code
+                && let KeyCode::Esc | KeyCode::Enter = key_event.code
+            {
+                req.submit(&app.auth_agent).await?;
+            }
+        }
+        FocusedBlock::DisplayPasskey => {
+            if let Some(req) = &mut app.requests.display_passkey
+                && key_event.code == KeyCode::Esc
+            {
+                req.cancel(&app.auth_agent).await?;
+            }
+        }
+
         _ => {
             match key_event.code {
                 // Exit the app
@@ -191,10 +286,10 @@ pub async fn handle_key_events(
                     FocusedBlock::PairedDevices => {
                         if let Some(selected_controller) = app.controller_state.selected() {
                             let controller = &app.controllers[selected_controller];
-                            if controller.new_devices.is_empty() {
-                                app.focused_block = FocusedBlock::Adapter;
-                            } else {
+                            if controller.is_scanning.load(Ordering::Relaxed) {
                                 app.focused_block = FocusedBlock::NewDevices;
+                            } else {
+                                app.focused_block = FocusedBlock::Adapter;
                             }
                         }
                     }
@@ -209,10 +304,10 @@ pub async fn handle_key_events(
                     FocusedBlock::Adapter => {
                         if let Some(selected_controller) = app.controller_state.selected() {
                             let controller = &app.controllers[selected_controller];
-                            if controller.new_devices.is_empty() {
-                                app.focused_block = FocusedBlock::PairedDevices;
-                            } else {
+                            if controller.is_scanning.load(Ordering::Relaxed) {
                                 app.focused_block = FocusedBlock::NewDevices;
+                            } else {
+                                app.focused_block = FocusedBlock::PairedDevices;
                             }
                             app.reset_devices_state();
                         }
@@ -717,33 +812,6 @@ pub async fn handle_key_events(
                             }
                         }
 
-                        FocusedBlock::PassKeyConfirmation => match key_event.code {
-                            KeyCode::Left | KeyCode::Char('h') => {
-                                if !app.pairing_confirmation.confirmed {
-                                    app.pairing_confirmation.confirmed = true;
-                                }
-                            }
-                            KeyCode::Right | KeyCode::Char('l') => {
-                                if app.pairing_confirmation.confirmed {
-                                    app.pairing_confirmation.confirmed = false;
-                                }
-                            }
-
-                            KeyCode::Enter => {
-                                app.pairing_confirmation
-                                    .user_confirmation_sender
-                                    .send(app.pairing_confirmation.confirmed)
-                                    .await?;
-                                app.pairing_confirmation
-                                    .display
-                                    .store(false, Ordering::Relaxed);
-                                app.focused_block = FocusedBlock::PairedDevices;
-                                app.pairing_confirmation.message = None;
-                            }
-
-                            _ => {}
-                        },
-
                         _ => {}
                     }
                 }
